@@ -46,14 +46,14 @@
 ;;; of value forms, a list of the single store-value form, a storing function,
 ;;; and an accessing function.
 (declaim (ftype (function (t &optional lexenv-designator))
-                sb-xc:get-setf-expansion))
-(defun sb-xc:get-setf-expansion (form &optional environment)
+                get-setf-expansion))
+(defun get-setf-expansion (form &optional environment)
   "Return five values needed by the SETF machinery: a list of temporary
    variables, a list of values with which to fill them, a list of temporaries
    for the new values, the setting function, and the accessing function."
   (named-let retry ((form form))
     (labels ((newvals (count)
-               (let ((sb-xc:*gensym-counter* 1))
+               (let ((*gensym-counter* 1))
                  (make-gensym-list count "NEW")))
              ;; Produce the expansion of a SETF form that calls either
              ;; #'(SETF name) or an inverse given by short form DEFSETF.
@@ -68,7 +68,7 @@
           (multiple-value-bind (expansion expanded)
               ;; Previously this called %MACROEXPAND, but the two operations
               ;; are equivalent on atoms, so do the one that is "less".
-              (sb-xc:macroexpand-1 form environment)
+              (macroexpand-1 form environment)
             (if expanded
                 (retry expansion)
                 (let ((vals (newvals 1)))
@@ -76,12 +76,18 @@
           (let ((fname (car form)))
             ;; Local functions inhibit global SETF methods.
             (unless (sb-c::fun-locally-defined-p fname environment)
+              ;; There are 3 possibilities for the expander:
+              ;;  #<fun>                    - define-setf-expander
+              ;;  (symbol doc . source-loc) - defsetf short form
+              ;;  (integer . #<fun>)        - defsetf long form
               (awhen (info :setf :expander fname)
                 (return-from retry
                   (typecase it
-                    (symbol ; short DEFSETF
-                     (call `(,it) (lambda (new args) `(,@args ,new))))
-                    (list   ; long DEFSETF
+                    (function ; DEFINE-SETF-EXPANDER
+                     (funcall it form environment))
+                    ((cons symbol) ; short form DEFSETF
+                     (call `(,(car it)) (lambda (new args) `(,@args ,new))))
+                    (t ; long form DEFSETF
                      (binding* ((newvals (newvals (car it)))
                                 (expander (the function (cdr it)))
                                 ((tempvars tempvals call-args)
@@ -91,18 +97,14 @@
                                   (or #+sb-xc (%fun-lambda-list expander)))))
                        (values tempvars tempvals newvals
                                (apply expander call-args environment newvals)
-                               `(,fname ,@call-args))))
-                    ;; DEFINE-SETF-EXPANDER
-                    (function (funcall it form environment)))))
+                               `(,fname ,@call-args)))))))
               (awhen (transformable-struct-setf-p form environment)
                 (let ((instance (make-symbol "OBJ"))
                       (vals (newvals 1)))
                   (return-from retry
                     (values (list instance) (list (cadr form)) vals
-                            (slot-access-transform
-                             :setf (list instance (car vals)) it)
-                            (slot-access-transform
-                             :read (list instance) it))))))
+                            (slot-access-transform :setf (list instance (car vals)) it)
+                            (slot-access-transform :read (list instance) it))))))
             (multiple-value-bind (expansion expanded)
                 (%macroexpand-1 form environment)
               (if expanded
@@ -178,15 +180,14 @@
             ;; NIL is not a valid setf inverse name, for two reasons:
             ;;  1. you can't define a function named NIL,
             ;;  2. (DEFSETF THING () ...) is the long form DEFSETF syntax.
-            (when (typep inverse '(and symbol (not null)))
-              (return-from setf `(,inverse ,@(cdr place) ,value-form))))
+            (when (typep inverse '(cons symbol))
+              (return-from setf `(,(car inverse) ,@(cdr place) ,value-form))))
           (awhen (transformable-struct-setf-p place env)
             (return-from setf
-              (slot-access-transform
-               :setf (list (cadr place) value-form) it)))))
+              (slot-access-transform :setf (list (cadr place) value-form) it)))))
 
       (multiple-value-bind (temps vals newval setter)
-          (sb-xc:get-setf-expansion place env)
+          (get-setf-expansion place env)
         (car (gen-let* (mapcar #'list temps vals)
                        (gen-mv-bind newval value-form (forms-list setter)))))))
 
@@ -202,7 +203,7 @@
   (collect ((let*-bindings) (mv-bindings) (setters) (getters))
     (dolist (arg (butlast args))
       (multiple-value-bind (temps subforms store-vars setter getter)
-          (sb-xc:get-setf-expansion arg env)
+          (get-setf-expansion arg env)
         (let*-bindings (mapcar #'list  temps subforms))
         (mv-bindings store-vars)
         (setters setter)
@@ -239,7 +240,7 @@
                (when (and (not (symbolp place)) (eq operator 'psetq))
                  (%program-error "Place ~S in PSETQ is not a SYMBOL" place))
                (multiple-value-bind (temps vals stores setter)
-                   (sb-xc:get-setf-expansion place env)
+                   (get-setf-expansion place env)
                  (let*-bindings (mapcar #'list temps vals))
                  (mv-bindings (cons stores value-form))
                  (setters setter))))
@@ -276,7 +277,7 @@
     (collect ((let*-bindings) (mv-bindings) (setters) (getters))
       (dolist (arg args)
         (multiple-value-bind (temps subforms store-vars setter getter)
-            (sb-xc:get-setf-expansion arg env)
+            (get-setf-expansion arg env)
           (let*-bindings (mapcar #'list temps subforms))
           (mv-bindings store-vars)
           (setters setter)
@@ -321,7 +322,7 @@
   (if (symbolp (setq place (macroexpand-for-setf place env)))
       `(prog1 (car ,place) (setq ,place (cdr ,place)))
       (multiple-value-bind (temps vals stores setter getter)
-          (sb-xc:get-setf-expansion place env)
+          (get-setf-expansion place env)
         (let ((list (copy-symbol 'list))
               (ret (copy-symbol 'car)))
           `(let* (,@(mapcar #'list temps vals)
@@ -338,7 +339,7 @@
   remove the property specified by the indicator. Returns T if such a
   property was present, NIL if not."
   (multiple-value-bind (temps vals newval setter getter)
-      (sb-xc:get-setf-expansion place env)
+      (get-setf-expansion place env)
     (let* ((flag (make-symbol "FLAG"))
            (body `(multiple-value-bind (,(car newval) ,flag)
               ;; See ANSI 5.1.3 for why we do out-of-order evaluation
@@ -380,7 +381,7 @@
          (if (symbolp (setq place (macroexpand-for-setf place env)))
              `(setq ,place (,operator ,delta ,place))
              (multiple-value-bind (dummies vals newval setter getter)
-                 (sb-xc:get-setf-expansion place env)
+                 (get-setf-expansion place env)
                `(let* (,@(mapcar #'list dummies vals)
                        (,(car newval) (,operator ,delta ,getter))
                        ,@(cdr newval))
@@ -397,7 +398,7 @@
 
 ;;;; DEFINE-MODIFY-MACRO stuff
 
-(sb-xc:defmacro sb-xc:define-modify-macro (name lambda-list function &optional doc-string)
+(sb-xc:defmacro define-modify-macro (name lambda-list function &optional doc-string)
   "Creates a new read-modify-write macro like PUSH or INCF."
   (check-designator name define-modify-macro)
   (binding* (((nil required optional rest)
@@ -417,55 +418,51 @@
 
 ;;;; DEFSETF
 
-(eval-when (#-sb-xc :compile-toplevel :load-toplevel :execute)
-  ;;; Assign SETF macro information for NAME, making all appropriate checks.
-  (defun %defsetf (name expander &optional doc)
-    (declare (ignorable doc))
-    (with-single-package-locked-error
-        (:symbol name "defining a setf-expander for ~A"))
-    (let ((setf-fn-name `(setf ,name)))
-      (multiple-value-bind (where-from present-p)
-          (info :function :where-from setf-fn-name)
-        ;; One might think that :DECLARED merits a style warning, but SBCL
-        ;; provides ~58 standard accessors as both (SETF F) and a macro.
-        ;; So allow the user to declaim an FTYPE and we'll hush up.
-        ;; What's good for the the goose is good for the gander.
-        (case where-from
-          (:assumed
-           ;; This indicates probable user error. Compilation assumed something
-           ;; to be functional; a macro says otherwise. Because :where-from's
-           ;; default can be :assumed, PRESENT-P disambiguates "defaulted" from
-           ;; "known" to have made an existence assumption.
-           (when present-p
-             (warn "defining setf macro for ~S when ~S was previously ~
+;;; Assign SETF macro information for NAME, making all appropriate checks.
+(defun %defsetf (name expander)
+  (with-single-package-locked-error
+      (:symbol name "defining a setf-expander for ~A"))
+  (let ((setf-fn-name `(setf ,name)))
+    (multiple-value-bind (where-from present-p)
+        (info :function :where-from setf-fn-name)
+      ;; One might think that :DECLARED merits a style warning, but SBCL
+      ;; provides ~58 standard accessors as both (SETF F) and a macro.
+      ;; So allow the user to declaim an FTYPE and we'll hush up.
+      ;; What's good for the the goose is good for the gander.
+      (case where-from
+        (:assumed
+         ;; This indicates probable user error. Compilation assumed something
+         ;; to be functional; a macro says otherwise. Because :where-from's
+         ;; default can be :assumed, PRESENT-P disambiguates "defaulted" from
+         ;; "known" to have made an existence assumption.
+         (when present-p
+           (warn "defining setf macro for ~S when ~S was previously ~
              treated as a function" name setf-fn-name)))
-          ;; This is a useless and unavoidable warning during self-build.
-          ;; cf. similar disabling of warning in WARN-IF-SETF-MACRO.
-          #-sb-xc-host
-          (:defined
-           ;; Somebody defined (SETF F) but then also said F has a macro.
-           ;; A soft warning seems appropriate because in this case it's
-           ;; at least in theory not wrong to call the function.
-           ;; The user can declare an FTYPE if both things are intentional.
-           (style-warn "defining setf macro for ~S when ~S is also defined"
-                       name setf-fn-name)))))
-    (setf (info :setf :expander name) expander)
-    #-sb-xc-host
-    (when doc
-      (setf (documentation name 'setf) doc))
-    name))
+        ;; This is a useless and unavoidable warning during self-build.
+        ;; cf. similar disabling of warning in WARN-IF-SETF-MACRO.
+        #-sb-xc-host
+        (:defined
+         ;; Somebody defined (SETF F) but then also said F has a macro.
+         ;; A soft warning seems appropriate because in this case it's
+         ;; at least in theory not wrong to call the function.
+         ;; The user can declare an FTYPE if both things are intentional.
+         (style-warn "defining setf macro for ~S when ~S is also defined"
+                     name setf-fn-name)))))
+  (setf (info :setf :expander name) expander)
+  name)
 
 ;;; This is pretty broken if there are keyword arguments (lp#1452947)
 ;;; but the bug seems to be due to irreconcilable problems in the spec.
 ;;; Everybody seems to interpret the spec the way we do though.
-(sb-xc:defmacro sb-xc:defsetf (access-fn &rest rest)
+(sb-xc:defmacro defsetf (access-fn &rest rest)
   "Associates a SETF update function or macro with the specified access
   function or macro. The format is complex. See the manual for details."
   (check-designator access-fn defsetf "access-function")
   (typecase rest
     ((cons (and symbol (not null)) (or null (cons string null)))
      `(eval-when (:load-toplevel :compile-toplevel :execute)
-        (%defsetf ',access-fn ',(car rest) ,@(cdr rest))))
+        (%defsetf ',access-fn
+                  (list* ',(car rest) ,(cadr rest) (sb-c:source-location)))))
     ((cons list (cons list))
      (destructuring-bind (lambda-list (&rest stores) &body body) rest
        (binding* (((llks req opt rest key aux env)
@@ -485,15 +482,14 @@
          `(eval-when (:compile-toplevel :load-toplevel :execute)
             (%defsetf ',access-fn
                       (cons ,(length stores)
-                            (named-lambda (%defsetf ,access-fn)
-                                          (,subforms ,env-var ,@stores)
+                            (named-lambda (%defsetf ,access-fn) (,subforms ,env-var ,@stores)
                               (declare (sb-c::lambda-list ,lambda-list))
+                              ,@(if doc (list doc))
                               ,@(if outer-decls (list outer-decls))
                               ,@(unless env `((declare (ignore ,env-var))))
                               (apply (lambda ,lambda-list
                                        ,@inner-decls (block ,access-fn ,@forms))
-                                     ,subforms)))
-                      ,@(and doc `(,doc)))))))
+                                     ,subforms))))))))
     (t
      (error "Ill-formed DEFSETF for ~S" access-fn))))
 
@@ -508,7 +504,8 @@
 ;;
 (defun collect-setf-temps (sexprs environment name-hints)
   (labels ((next-name-hint ()
-             (let ((sym (pop name-hints))) ; OK if list was nil
+             ;; OK if list was nil or :UNKNOWN
+             (let ((sym (and (listp name-hints) (pop name-hints))))
                (case sym
                  (&optional (next-name-hint))
                  ((&key &rest) (setq name-hints nil))
@@ -520,7 +517,7 @@
       (let ((mask 0) (bit 1))
         (dolist (form sexprs (values (temp-vars) (temp-vals) (call-arguments)
                                      mask))
-          (call-arguments (if (sb-xc:constantp form environment)
+          (call-arguments (if (constantp form environment)
                               (progn (next-name-hint) form) ; Skip one hint.
                               (let ((temp (nice-tempname form)))
                                 (setq mask (logior mask bit))
@@ -542,7 +539,7 @@
      (binding* (((before-temps before-vals before-args)
                  (collect-setf-temps before-arg-forms environment name-hints))
                 ((place-temps place-subforms stores setter getter)
-                 (sb-xc:get-setf-expansion place environment))
+                 (get-setf-expansion place environment))
                 ((after-temps after-vals after-args)
                  (if after-args-bindp
                      (collect-setf-temps after-arg-forms environment name-hints)
@@ -559,7 +556,7 @@
                            (every (lambda (x)
                                     (or (member x place-temps)
                                         (eq x newval-temp)
-                                        (sb-xc:constantp x environment)))
+                                        (constantp x environment)))
                                   (cdr setter)))))
          (setq newval-binding nil
                setter (substitute compute newval-temp setter)))
@@ -575,17 +572,12 @@
 ;;;; DEFMACRO DEFINE-SETF-EXPANDER and various DEFINE-SETF-EXPANDERs
 
 ;;; DEFINE-SETF-EXPANDER is a lot like DEFMACRO.
-(sb-xc:defmacro sb-xc:define-setf-expander (access-fn lambda-list &body body)
+(sb-xc:defmacro define-setf-expander (access-fn lambda-list &body body)
   "Syntax like DEFMACRO, but creates a setf expander function. The body
   of the definition must be a form that returns five appropriate values."
   (check-designator access-fn define-setf-expander "access-function")
-  (multiple-value-bind (def doc)
-      ;; Perhaps it would be more elegant to keep the docstring attached
-      ;; to the expander function, as for CAS?
-      (make-macro-lambda `(setf-expander ,access-fn) lambda-list body
-                         'sb-xc:define-setf-expander access-fn
-                         :doc-string-allowed :external)
-    ;; Maybe kill docstring, but only under the cross-compiler.
-    #+(and (not sb-doc) sb-xc-host) (setq doc nil)
-    `(eval-when (:compile-toplevel :load-toplevel :execute)
-       (%defsetf ',access-fn ,def ,@(and doc `(,doc))))))
+  `(eval-when (:compile-toplevel :load-toplevel :execute)
+     (%defsetf ',access-fn
+               ,(make-macro-lambda `(setf-expander ,access-fn) lambda-list body
+                                   'define-setf-expander access-fn
+                                   :doc-string-allowed :internal))))
