@@ -55,6 +55,16 @@
   (def loadw ldr word-shift)
   (def storew str word-shift))
 
+(defmacro storew-pair (object1 offset1 object2 offset2
+                       base)
+  (assert (= (eval offset1) (1- (eval offset2))))
+  `(inst stp ,object1 ,object2 (@ ,base (* ,(eval offset1) n-word-bytes))))
+
+(defmacro loadw-pair (object1 offset1 object2 offset2
+                       base)
+  (assert (= (eval offset1) (1- (eval offset2))))
+  `(inst ldp ,object1 ,object2 (@ ,base (* ,(eval offset1) n-word-bytes))))
+
 (defmacro load-symbol (reg symbol)
   (once-only ((reg reg) (symbol symbol))
     `(inst add ,reg null-tn (add-sub-immediate  (static-symbol-offset ,symbol)))))
@@ -80,45 +90,22 @@
                          #+big-endian `(+ ,n-offset (1- n-word-bytes))))
       `(inst ldrb ,n-target (@ ,n-source ,target-offset)))))
 
-;;; Macros to handle the fact that our stack pointer isn't actually in
-;;; a register (or won't be, by the time we're done).
-
-;;; Macros to handle the fact that we cannot use the machine native call and
-;;; return instructions.
-
-(defmacro lisp-jump (function lip)
-  "Jump to the lisp lip LIP."
-  `(let ((function ,function)
-         (lip ,lip))
+(defmacro lisp-jump (lip)
+  `(let ((lip ,lip))
      (aver (sc-is lip interior-reg))
-     (inst add lip function
-           (- (ash simple-fun-insts-offset word-shift)
-              fun-pointer-lowtag))
+     (inst add lip lip 4)
      (inst br lip)))
 
-(defmacro lisp-return (function lip return-style)
+(defmacro lisp-return (lip return-style)
   "Return to RETURN-PC."
-  `(let* ((function ,function)
-          (lip ,lip))
-     ;; Indicate a single-valued return by clearing all of the status
-     ;; flags, or a multiple-valued return by setting all of the status
-     ;; flags.
+  `(let* ((lip ,lip))
      (aver (sc-is lip interior-reg))
+     ;; Indicate a single-valued return by clearing the Z flag
      ,@(ecase return-style
-         (:single-value '((inst msr :nzcv zr-tn)))
-         (:multiple-values '((inst orr tmp-tn zr-tn #xf0000000)
-                             (inst msr :nzcv tmp-tn)))
+         (:single-value '((inst cmp null-tn 0)))
+         (:multiple-values '((inst cmp zr-tn zr-tn)))
          (:known))
-     (inst sub lip function (- other-pointer-lowtag 8))
      (inst ret lip)))
-
-(defmacro emit-return-pc (label)
-  "Emit a return-pc header word.  LABEL is the label to use for this return-pc."
-  `(progn
-     (emit-alignment n-lowtag-bits)
-     (emit-label ,label)
-     (inst lra-header-word)))
-
 
 ;;;; Stack TN's
 
@@ -149,7 +136,7 @@
   (once-only ((n-reg reg)
               (n-stack reg-or-stack))
     `(sc-case ,n-reg
-       ((any-reg descriptor-reg)
+       ((any-reg descriptor-reg interior-reg)
         (sc-case ,n-stack
           ((any-reg descriptor-reg)
            (move ,n-reg ,n-stack))
@@ -174,37 +161,30 @@
 ;;; P-A FLAG-TN is also acceptable here.
 
 #+gencgc
-(defun allocation-tramp (type alloc-tn size back-label return-in-tmp lip)
-  (unless (eq size tmp-tn)
-    (inst mov tmp-tn size))
+(defun allocation-tramp (type alloc-tn size back-label lip)
+  (if (integerp size)
+      (load-immediate-word tmp-tn size)
+      (inst mov tmp-tn size))
   (let ((asm-routine (if (eq type 'list) 'list-alloc-tramp 'alloc-tramp)))
     (load-inline-constant alloc-tn `(:fixup ,asm-routine :assembly-routine) lip))
   (inst blr alloc-tn)
-  (unless return-in-tmp
-    (move alloc-tn tmp-tn))
   (inst b back-label))
 
+;;; Leaves the untagged pointer in TMP-TN,
+;;; Allowing it to be used with STP later.
 (defun allocation (type size lowtag result-tn
-                      &key flag-tn
-                           stack-allocate-p
-                           (lip (if stack-allocate-p nil (missing-arg))))
+                   &key flag-tn
+                        stack-allocate-p
+                        (lip (if stack-allocate-p nil (missing-arg)))
+                        overflow)
   (declare (ignorable type lip))
   ;; Normal allocation to the heap.
   (if stack-allocate-p
       (assemble ()
-        (move result-tn csp-tn)
-        (inst tst result-tn lowtag-mask)
-        (inst b :eq ALIGNED)
-        (inst add result-tn result-tn n-word-bytes)
-        ALIGNED
-        (inst add csp-tn result-tn (add-sub-immediate size))
-        ;; :ne is from TST above, this needs to be done after the
-        ;; stack pointer has been stored.
-        (inst b :eq ALIGNED2)
-        (storew zr-tn result-tn -1 0)
-        ALIGNED2
-        (when lowtag
-          (inst add result-tn result-tn lowtag)))
+        (inst add tmp-tn csp-tn lowtag-mask)
+        (inst and tmp-tn tmp-tn (lognot lowtag-mask))
+        (inst add csp-tn tmp-tn (add-sub-immediate size result-tn))
+        (inst add result-tn tmp-tn lowtag))
       #-gencgc
       (progn
         (load-symbol-value flag-tn *allocation-pointer*)
@@ -212,42 +192,41 @@
         (inst add flag-tn flag-tn (add-sub-immediate size))
         (store-symbol-value flag-tn *allocation-pointer*))
       #+gencgc
-      (let ((alloc (gen-label)) (back-from-alloc (gen-label)))
+      (let ((alloc (gen-label))
+            #+sb-thread (tlab (if (eq type 'list) thread-cons-tlab-slot thread-mixed-tlab-slot))
+            #-sb-thread (region (if (eq type 'list) cons-region mixed-region))
+            (back-from-alloc (gen-label)))
         #-sb-thread
         (progn
           ;; load-pair can't base off null-tn because the displacement
           ;; has to be a multiple of 8
-          (load-immediate-word flag-tn static-space-start)
-          (inst ldp result-tn flag-tn (@ flag-tn (* 2 n-word-bytes))))
+          (load-immediate-word flag-tn region)
+          (inst ldp result-tn flag-tn (@ flag-tn 0)))
         #+sb-thread
-        (inst ldp result-tn flag-tn (@ thread-tn (* n-word-bytes thread-alloc-region-slot)))
-        (setf size (add-sub-immediate size))
-        (inst add result-tn result-tn size)
+        (inst ldp tmp-tn flag-tn (@ thread-tn (* n-word-bytes tlab)))
+        (inst add result-tn tmp-tn (add-sub-immediate size result-tn))
         (inst cmp result-tn flag-tn)
         (inst b :hi ALLOC)
-        #-sb-thread (inst str result-tn (@ null-tn  (- (+ static-space-start (* 2 n-word-bytes))
-                                                       nil-value)))
-        #+sb-thread (storew result-tn thread-tn thread-alloc-region-slot)
-        ;; alloc_tramp uses tmp-tn for returning the result,
-        ;; save on a move when possible
-        (inst sub (if lowtag tmp-tn result-tn) result-tn size)
+        #-sb-thread (inst str result-tn (@ null-tn (load-store-offset (- region nil-value))))
+        #+sb-thread (storew result-tn thread-tn tlab)
+
         (emit-label BACK-FROM-ALLOC)
-        (when lowtag
-          (inst add result-tn tmp-tn lowtag))
+        (inst add result-tn tmp-tn lowtag)
         (assemble (:elsewhere)
           (emit-label ALLOC)
-          (allocation-tramp type
-                            result-tn
-                            size
-                            BACK-FROM-ALLOC
-                            ;; see the comment above aboout alloc_tramp
-                            lowtag
-                            lip)))))
+          (if overflow
+              (funcall overflow)
+              (allocation-tramp type
+                                result-tn
+                                size
+                                BACK-FROM-ALLOC
+                                lip))))))
 
 (defmacro with-fixed-allocation ((result-tn flag-tn type-code size
                                             &key (lowtag other-pointer-lowtag)
                                                  stack-allocate-p
-                                                 (lip (missing-arg)))
+                                                 (lip (missing-arg))
+                                                 (store-type-code t))
                                  &body body)
   "Do stuff to allocate an other-pointer object of fixed Size with a single
   word header having the specified Type-Code.  The result is placed in
@@ -258,27 +237,51 @@
               (type-code type-code) (size size) (lowtag lowtag)
               (stack-allocate-p stack-allocate-p)
               (lip lip))
-    `(pseudo-atomic (,flag-tn)
+    `(pseudo-atomic (,flag-tn :sync ,type-code
+                     :elide-if ,stack-allocate-p)
        (allocation nil (pad-data-block ,size) ,lowtag ,result-tn
                    :flag-tn ,flag-tn
                    :stack-allocate-p ,stack-allocate-p
                    :lip ,lip)
-       (when ,type-code
-         (load-immediate-word ,flag-tn (compute-object-header ,size ,type-code))
-         (storew ,flag-tn ,result-tn 0 ,lowtag))
+       (load-immediate-word ,flag-tn (compute-object-header ,size ,type-code))
+       ,@(and store-type-code
+              `((storew ,flag-tn ,result-tn 0 ,lowtag)))
        ,@body)))
 
 ;;;; Error Code
+;;;; BRK accepts a 16-bit immediate
+;;;; Encode the error kind in the first byte.
+;;;; If KIND is ERROR-TRAP, then add CODE to that first byte.
+;;;; Otherwise CODE goes into the byte following BRK.
+;;;; The arguments are normally encoded by ENCODE-INTERNAL-ERROR-ARGS,
+;;;; except for the first argument if it's a descriptor-reg or
+;;;; any-reg, then it goes into the second byte of the BRK instruction
+;;;; immediate.
+;;;; Otherwise that second byte is 31 (the ZR register).
 (defun emit-error-break (vop kind code values)
   (assemble ()
     (when vop
       (note-this-location vop :internal-error))
-    ;; Encode both kind and code as an argument to BRK
-    (inst brk (dpb code (byte 8 8) kind))
-    ;; NARGS is implicitely assumed for invalid-arg-count
-    (unless (= kind invalid-arg-count-trap)
-      (encode-internal-error-args values)
-      (emit-alignment 2))))
+    (cond ((= kind invalid-arg-count-trap)
+           ;; NARGS is implicitly assumed for invalid-arg-count
+           (inst brk kind)
+           (return-from emit-error-break))
+          (t
+           (let ((first-value (car values)))
+             (inst brk (dpb (cond ((and (tn-p first-value)
+                                        (sc-is first-value descriptor-reg any-reg))
+                                   (pop values)
+                                   (tn-offset first-value))
+                                  (t
+                                   zr-offset))
+                            (byte 8 8)
+                            (if (= kind error-trap)
+                                (+ kind code)
+                                kind)))
+             (unless (= kind error-trap)
+               (inst byte code)))))
+    (encode-internal-error-args values)
+    (emit-alignment 2)))
 
 (defun generate-error-code (vop error-code &rest values)
   "Generate-Error-Code Error-code Value*
@@ -302,40 +305,42 @@
                            other-pointer-lowtag)))))
 
 ;;; handy macro for making sequences look atomic
-(defmacro pseudo-atomic ((flag-tn) &body forms)
-  #+sb-safepoint-strictly
+(defmacro pseudo-atomic ((flag-tn &key elide-if (sync t)) &body forms)
+  (declare (ignorable sync))
+  #+sb-safepoint
   `(progn ,@forms (emit-safepoint))
-  #-sb-safepoint-strictly
+  #-sb-safepoint
   `(progn
-     (without-scheduling ()
-       #-sb-thread
-       (store-symbol-value csp-tn *pseudo-atomic-atomic*)
-       #+sb-thread
-       (inst str (32-bit-reg null-tn)
-             (@ thread-tn
-                (* n-word-bytes thread-pseudo-atomic-bits-slot))))
+     (unless ,elide-if
+       (without-scheduling ()
+         #-sb-thread
+         (store-symbol-value csp-tn *pseudo-atomic-atomic*)
+         #+sb-thread
+         (inst str (32-bit-reg null-tn)
+               (@ thread-tn
+                  (* n-word-bytes thread-pseudo-atomic-bits-slot)))))
      (assemble ()
        ,@forms)
-     (without-scheduling ()
-       #-sb-thread
-       (progn
-         (store-symbol-value null-tn *pseudo-atomic-atomic*)
-         (load-symbol-value ,flag-tn *pseudo-atomic-interrupted*))
-       #+sb-thread
-       (progn
-         (inst dmb)
-         (inst str (32-bit-reg zr-tn)
-               (@ thread-tn
-                  (* n-word-bytes thread-pseudo-atomic-bits-slot)))
-         (inst ldr (32-bit-reg ,flag-tn)
-               (@ thread-tn
-                  (+ (* n-word-bytes thread-pseudo-atomic-bits-slot) 4))))
-       (let ((not-interrputed (gen-label)))
-         (inst cbz ,flag-tn not-interrputed)
-         (inst brk pending-interrupt-trap)
-         (emit-label not-interrputed))
-       #+sb-safepoint
-       (emit-safepoint))))
+     (unless ,elide-if
+       (without-scheduling ()
+         #-sb-thread
+         (progn
+           (store-symbol-value null-tn *pseudo-atomic-atomic*)
+           (load-symbol-value ,flag-tn *pseudo-atomic-interrupted*))
+         #+sb-thread
+         (progn
+           (when ,sync
+            (inst dmb))
+           (inst str (32-bit-reg zr-tn)
+                 (@ thread-tn
+                    (* n-word-bytes thread-pseudo-atomic-bits-slot)))
+           (inst ldr (32-bit-reg ,flag-tn)
+                 (@ thread-tn
+                    (+ (* n-word-bytes thread-pseudo-atomic-bits-slot) 4))))
+         (let ((not-interrputed (gen-label)))
+           (inst cbz ,flag-tn not-interrputed)
+           (inst brk pending-interrupt-trap)
+           (emit-label not-interrputed))))))
 
 ;;;; memory accessor vop generators
 
@@ -369,11 +374,9 @@
      (:policy :fast-safe)
      (:args (object :scs (descriptor-reg))
             (index :scs (any-reg immediate))
-            (value :scs ,scs :target result))
+            (value :scs (,@scs zero)))
      (:arg-types ,type tagged-num ,el-type)
      (:temporary (:scs (interior-reg)) lip)
-     (:results (result :scs ,scs))
-     (:result-types ,el-type)
      (:generator 2
        (sc-case index
          (immediate
@@ -382,8 +385,7 @@
                                         ,lowtag)))))
          (t
           (inst add lip object (lsl index (- word-shift n-fixnum-tag-bits)))
-          (storew value lip ,offset ,lowtag)))
-       (move result value))))
+          (storew value lip ,offset ,lowtag))))))
 
 (defmacro define-partial-reffer (name type size signed offset lowtag scs
                                  el-type &optional translate)
@@ -435,12 +437,14 @@
      (:policy :fast-safe)
      (:args (object :scs (descriptor-reg))
             (index :scs (any-reg unsigned-reg immediate))
-            (value :scs ,scs :target result))
+            (value :scs ,scs
+                   :load-if (not (and (sc-is value immediate)
+                                      (eql (tn-value value) 0)))))
      (:arg-types ,type tagged-num ,el-type)
      (:temporary (:scs (interior-reg)) lip)
-     (:results (result :scs ,scs))
-     (:result-types ,el-type)
      (:generator 5
+       (when (sc-is value immediate)
+         (setf value zr-tn))
        ,@(multiple-value-bind (op shift)
              (ecase size
                (:byte
@@ -467,8 +471,7 @@
                                              (asr index (- shift))
                                              (lsl index shift)))
                     (inst ,op
-                          ,value (@ lip (- (* ,offset n-word-bytes) ,lowtag)))))))))
-       (move result value))))
+                          ,value (@ lip (- (* ,offset n-word-bytes) ,lowtag))))))))))))
 
 (defun load-inline-constant (dst value &optional lip)
   (destructuring-bind (size . label) (register-inline-constant value)
@@ -492,18 +495,12 @@
 
 (defmacro load-tl-symbol-value (reg symbol)
   #+sb-thread
-  `(let ((reg ,reg))
-     (load-symbol tmp-tn ',symbol)
-     (inst ldr (32-bit-reg tmp-tn) (tls-index-of tmp-tn))
-     (inst ldr reg (@ thread-tn tmp-tn)))
+  `(inst ldr ,reg (@ thread-tn ,(info :variable :wired-tls symbol)))
   #-sb-thread
   `(load-symbol-value ,reg ,symbol))
 
 (defmacro store-tl-symbol-value (reg symbol)
   #+sb-thread
-  `(let ((reg ,reg))
-     (load-symbol tmp-tn ',symbol)
-     (inst ldr (32-bit-reg tmp-tn) (tls-index-of tmp-tn))
-     (inst str reg (@ thread-tn tmp-tn)))
+  `(inst str ,reg (@ thread-tn ,(info :variable :wired-tls symbol)))
   #-sb-thread
   `(store-symbol-value ,reg ,symbol))
