@@ -27,7 +27,8 @@
 
 ;;; Report as we try each transform? (either source or IR)
 ;;; Bind to T to see the ones that didn't abort,
-;;; or :ALL if you want to see each attempted transform.
+;;; :ALL if you want to see each attempted transform.
+;;; :DERIVE-TYPE for newly derived types of combinations.
 (defvar *show-transforms-p* nil)
 
 (declaim (inline source-form-has-path-p))
@@ -38,18 +39,29 @@
   (when (source-form-has-path-p form)
     (gethash form *source-paths*)))
 
-(defvar *transforming* nil)
+(defvar *transforming* 0)
+(defvar *inlining* 0)
 
 (defun ensure-source-path (form)
-  (or (get-source-path form)
-      (if (and *transforming*
-               (not (memq 'transformed *current-path*)))
-          ;; Don't hide all the transformed paths, since we might want
-          ;; to look at the final form for error reporting.
-          (list* (simplify-source-path-form form)
-                 'transformed *current-path*)
-          (cons (simplify-source-path-form form)
-                *current-path*))))
+  (flet ((level> (value kind)
+           (let ((x (memq kind *current-path*)))
+             (> value
+                (if x
+                    (cadr x)
+                    0)))))
+    (or (get-source-path form)
+        (cond ((level> *transforming* 'transformed)
+               ;; Don't hide all the transformed paths, since we might want
+               ;; to look at the final form for error reporting.
+               (list* (simplify-source-path-form form)
+                      'transformed *transforming* *current-path*))
+              ;; Avoids notes about inlined code leaking out
+              ((level> *inlining* 'inlined)
+               (list* (simplify-source-path-form form)
+                      'inlined *inlining* *current-path*))
+              (t
+               (cons (simplify-source-path-form form)
+                     *current-path*))))))
 
 (defun simplify-source-path-form (form)
   (if (consp form)
@@ -93,11 +105,11 @@
 (defvar *current-path*)
 
 (defun call-with-current-source-form (thunk &rest forms)
-  (let ((*current-path* (when (and (some #'identity forms)
-                                   (boundp '*source-paths*))
-                          (or (some #'get-source-path forms)
-                              (when (boundp '*current-path*)
-                                *current-path*)))))
+  (let ((*current-path* (or (and (some #'identity forms)
+                                 (boundp '*source-paths*)
+                                 (some #'get-source-path forms))
+                            (and (boundp '*current-path*)
+                                 *current-path*))))
     (funcall thunk)))
 
 (defvar *derive-function-types* nil
@@ -156,7 +168,7 @@
     ;; If ANSWER is NIL, go for the global value
     (eq (or answer (info :function :inlinep name)) 'notinline)))
 
-#-sb-devel
+
 (declaim (start-block find-free-fun find-lexically-apparent-fun
                       ;; needed by ir1-translators
                       find-global-fun))
@@ -171,42 +183,46 @@
 ;;; Return a GLOBAL-VAR structure usable for referencing the global
 ;;; function NAME.
 (defun find-global-fun (name latep)
-  (unless (info :function :kind name)
-    (setf (info :function :kind name) :function)
-    (setf (info :function :where-from name) :assumed))
-  (let ((where (info :function :where-from name)))
-    (when (and (eq where :assumed)
-               ;; Slot accessors are defined just-in-time, if not already.
-               (not (typep name '(cons (eql sb-pcl::slot-accessor))))
-               ;; In the ordinary target Lisp, it's silly to report
-               ;; undefinedness when the function is defined in the
-               ;; running Lisp. But at cross-compile time, the current
-               ;; definedness of a function is irrelevant to the
-               ;; definedness at runtime, which is what matters.
-               #-sb-xc-host (not (fboundp name))
-               ;; LATEP is true when the user has indicated that
-               ;; late-late binding is desired by using eg. a quoted
-               ;; symbol -- in which case it makes little sense to
-               ;; complain about undefined functions.
-               (not latep))
-      (note-undefined-reference name :function))
-    (let ((ftype (global-ftype name))
-          (notinline (fun-lexically-notinline-p name)))
-      (make-global-var
-       :kind :global-function
-       :%source-name name
-       :type (if (or (eq where :declared)
-                     (and (not latep)
-                          (not notinline)
-                          *derive-function-types*))
-                 ftype
-                 (specifier-type 'function))
-       :defined-type (if (and (not latep) (not notinline))
-                         ftype
-                         (specifier-type 'function))
-       :where-from (if notinline
-                       where
-                       (maybe-defined-here name where))))))
+  (let ((kind (info :function :kind name)))
+    (unless kind
+      (setf (info :function :kind name) :function)
+      (setf (info :function :where-from name) :assumed))
+    (let ((where (info :function :where-from name)))
+      (when (and (eq where :assumed)
+                 ;; Slot accessors are defined just-in-time, if not already.
+                 (not (typep name '(cons (eql sb-pcl::slot-accessor))))
+                 ;; In the ordinary target Lisp, it's silly to report
+                 ;; undefinedness when the function is defined in the
+                 ;; running Lisp. But at cross-compile time, the current
+                 ;; definedness of a function is irrelevant to the
+                 ;; definedness at runtime, which is what matters.
+                 #-sb-xc-host (not (fboundp name))
+                 ;; LATEP is true when the user has indicated that
+                 ;; late-late binding is desired by using eg. a quoted
+                 ;; symbol -- in which case it makes little sense to
+                 ;; complain about undefined functions.
+                 (not latep))
+        (note-undefined-reference name :function))
+      (case kind
+        ((:macro :special-form)
+         (compiler-warn "~(~a~) ~s where a function is expected" kind name)))
+      (let ((ftype (global-ftype name))
+            (notinline (fun-lexically-notinline-p name)))
+        (make-global-var
+         :kind :global-function
+         :%source-name name
+         :type (if (or (eq where :declared)
+                       (and (not latep)
+                            (not notinline)
+                            *derive-function-types*))
+                   ftype
+                   (specifier-type 'function))
+         :defined-type (if (and (not latep) (not notinline))
+                           ftype
+                           (specifier-type 'function))
+         :where-from (if notinline
+                         where
+                         (maybe-defined-here name where)))))))
 
 ;;; Have some DEFINED-FUN-FUNCTIONALS of a FREE-FUNS entry become invalid?
 ;;; Drop 'em.
@@ -337,8 +353,6 @@
             (type (info :variable :type name))
             (where-from (info :variable :where-from name))
             (deprecation-state (deprecated-thing-p 'variable name)))
-        (when (and (eq kind :unknown) (not deprecation-state))
-          (note-undefined-reference name :variable))
         ;; For deprecated vars, warn about LET and LAMBDA bindings, SETQ, and ref.
         ;; Don't warn again if the name was already seen by the transform
         ;; of SYMBOL[-GLOBAL]-VALUE.
@@ -360,6 +374,8 @@
                        (type (type-specifier (info :variable :type name))))
                    `(macro . (the ,type ,expansion))))
                 (:constant
+                 (when (producing-fasl-file)
+                   (emit-load-time-constant-reference name))
                  (find-constant (symbol-value name) name))
                 (t
                  (make-global-var :kind kind
@@ -367,20 +383,37 @@
                                   :type type
                                   :where-from where-from)))))))
 
+;;; Return T if and only if OBJ's nature as an externalizable thing renders
+;;; it a leaf for dumping purposes. Symbols are leaflike despite havings slots
+;;; containing pointers; similarly (COMPLEX RATIONAL) and RATIO.
+(defun dumpable-leaflike-p (obj)
+  (or (sb-xc:typep obj '(or symbol number character
+                            ;; (ARRAY NIL) is not included in UNBOXED-ARRAY
+                            (or unboxed-array (array nil))
+                            system-area-pointer
+                            #+sb-simd-pack simd-pack
+                            #+sb-simd-pack-256 simd-pack-256))
+      (cl:typep obj 'debug-name-marker)
+      ;; STANDARD-OBJECT layouts use MAKE-LOAD-FORM, but all other layouts
+      ;; have the same status as symbols - composite objects but leaflike.
+      (and (typep obj 'wrapper) (not (layout-for-pcl-obj-p obj)))
+      ;; PACKAGEs are also leaflike.
+      (cl:typep obj 'package)
+      ;; The cross-compiler wants to dump CTYPE instances as leaves,
+      ;; but CLASSOIDs are excluded since they have a MAKE-LOAD-FORM method.
+      #+sb-xc-host (cl:typep obj '(and ctype (not classoid)))))
+
 ;;; Grovel over CONSTANT checking for any sub-parts that need to be
 ;;; processed with MAKE-LOAD-FORM. We have to be careful, because
 ;;; CONSTANT might be circular. We also check that the constant (and
 ;;; any subparts) are dumpable at all.
 (defun maybe-emit-make-load-forms (constant)
   (declare #-sb-xc-host (inline alloc-xset))
-  (dx-let ((xset (alloc-xset)))
+  (dx-let ((things-processed (alloc-xset)))
     (named-let grovel ((value constant))
-      ;; Unless VALUE is an object which which can't contain other objects,
-      ;; or was visited, or is acccessible in a named constant ...
       (unless (or (dumpable-leaflike-p value)
-                  (xset-member-p value xset)
-                  (gethash value (eql-constants *ir1-namespace*)))
-        (add-to-xset value xset)
+                  (xset-member-p value things-processed))
+        (add-to-xset value things-processed)
         ;; FIXME: shouldn't this be something like SB-XC:TYPECASE ?
         (typecase value
           (cons
@@ -402,6 +435,7 @@
           ((array t)
            (dotimes (i (array-total-size value))
              (grovel (row-major-aref value i))))
+          (opaque-box)
           (instance
            ;; In the target SBCL, we can dump any instance, but
            ;; in the cross-compilation host, %INSTANCE-FOO
@@ -425,28 +459,26 @@
             "Objects of type ~/sb-impl:print-type-specifier/ can't be dumped into fasl files."
             (type-of value)))))))
   (values))
-
-;;; A constant is trivially externalizable if it involves no INSTANCE types
-;;; or any un-dumpable object.
-(defun trivially-externalizable-p (constant)
-  (declare #-sb-xc-host (inline alloc-xset))
-  (dx-let ((xset (alloc-xset)))
-    (named-let ok ((value constant))
-      (if (or (dumpable-leaflike-p value) (xset-member-p value xset))
-          t
-          (progn
-            (add-to-xset value xset)
-            (typecase value
-             (cons (and (ok (car value)) (ok (cdr value))))
-             (vector
-              (dotimes (i (length value) t)
-                (unless (ok (aref value i)) (return nil))))
-             (array
-              (dotimes (i (array-total-size value) t)
-                (unless (ok (row-major-aref value i)) (return nil))))))))))
 
 ;;;; some flow-graph hacking utilities
 
+;; Bind *COMPILER-ERROR-BAILOUT* to a function that throws out of the
+;; body and converts a condition signalling form instead. The source
+;; form is converted to a string since it may contain arbitrary
+;; non-externalizable objects.
+(defmacro ir1-error-bailout ((start next result form) &body body)
+  (with-unique-names (skip condition)
+    `(block ,skip
+       (let ((,condition (catch 'ir1-error-abort
+                           (let ((*compiler-error-bailout*
+                                   (lambda (&optional e)
+                                     (throw 'ir1-error-abort e))))
+                             ,@body
+                             (return-from ,skip nil)))))
+         (ir1-convert ,start ,next ,result
+                      (make-compiler-error-form ,condition
+                                                ,form))))))
+
 ;;; This function sets up the back link between the node and the
 ;;; ctran which continues at it.
 (defun link-node-to-previous-ctran (node ctran)
@@ -491,6 +523,7 @@
     (setf (ctran-next prev) nil)
     (link-node-to-previous-ctran new prev)
     (use-ctran new temp)
+    (setf (ctran-source-path temp) (ctran-source-path prev))
     (link-node-to-previous-ctran old temp))
   (values))
 
@@ -605,65 +638,59 @@
          (frob)
          (frob)
          (setq trail (cdr trail)))))))
+
 
 ;;;; IR1-CONVERT, macroexpansion and special form dispatching
-(declaim (ftype (sfunction (ctran ctran (or lvar null) t)
-                           (values))
-                ir1-convert))
-(macrolet (;; Bind *COMPILER-ERROR-BAILOUT* to a function that throws
-           ;; out of the body and converts a condition signalling form
-           ;; instead. The source form is converted to a string since it
-           ;; may contain arbitrary non-externalizable objects.
-           (ir1-error-bailout ((start next result form) &body body)
-             (with-unique-names (skip condition)
-               `(block ,skip
-                  (let ((,condition (catch 'ir1-error-abort
-                                      (let ((*compiler-error-bailout*
-                                              (lambda (&optional e)
-                                                (throw 'ir1-error-abort e))))
-                                        ,@body
-                                        (return-from ,skip nil)))))
-                    (ir1-convert ,start ,next ,result
-                                 (make-compiler-error-form ,condition
-                                                           ,form)))))))
 
-  ;; Translate FORM into IR1. The code is inserted as the NEXT of the
-  ;; CTRAN START. RESULT is the LVAR which receives the value of the
-  ;; FORM to be translated. The translators call this function
-  ;; recursively to translate their subnodes.
-  ;;
-  ;; As a special hack to make life easier in the compiler, a LEAF
-  ;; IR1-converts into a reference to that LEAF structure. This allows
-  ;; the creation using backquote of forms that contain leaf
-  ;; references, without having to introduce dummy names into the
-  ;; namespace.
-  (defun ir1-convert (start next result form)
-    (let* ((*current-path* (ensure-source-path form))
-           (start (instrument-coverage start nil form)))
-      (ir1-error-bailout (start next result form)
-        (cond ((atom form)
-               (cond ((and (symbolp form) (not (keywordp form)))
-                      (ir1-convert-var start next result form))
-                     ((leaf-p form)
-                      (reference-leaf start next result form))
-                     (t
-                      (reference-constant start next result form))))
-              (t
-               (ir1-convert-functoid start next result form)))))
-    (values))
+(declaim (start-block ir1-convert ir1-convert-progn-body
+                      ir1-convert-combination-args reference-leaf
+                      reference-constant
+                      expand-compiler-macro
+                      maybe-reanalyze-functional))
 
-  ;; Generate a reference to a manifest constant, creating a new leaf
-  ;; if necessary.
-  (defun reference-constant (start next result value)
-    (declare (type ctran start next)
-             (type (or lvar null) result))
-    (ir1-error-bailout (start next result value)
-      (let* ((leaf (find-constant value))
-             (res (make-ref leaf)))
-        (push res (leaf-refs leaf))
-        (link-node-to-previous-ctran res start)
-        (use-continuation res next result)))
-    (values)))
+;;; Translate FORM into IR1. The code is inserted as the NEXT of the
+;;; CTRAN START. RESULT is the LVAR which receives the value of the
+;;; FORM to be translated. The translators call this function
+;;; recursively to translate their subnodes.
+;;;
+;;; As a special hack to make life easier in the compiler, a LEAF
+;;; IR1-converts into a reference to that LEAF structure. This allows
+;;; the creation using backquote of forms that contain leaf
+;;; references, without having to introduce dummy names into the
+;;; namespace.
+(defun ir1-convert (start next result form)
+  (declare (type ctran start next)
+           (type (or lvar null) result))
+  (let* ((*current-path* (ensure-source-path form))
+         (start (instrument-coverage start nil form)))
+    (ir1-error-bailout (start next result form)
+                       (cond ((atom form)
+                              (cond ((and (symbolp form) (not (keywordp form)))
+                                     (ir1-convert-var start next result form))
+                                    ((leaf-p form)
+                                     (reference-leaf start next result form))
+                                    (t
+                                     (reference-constant start next result form))))
+                             (t
+                              (ir1-convert-functoid start next result form)))))
+  (values))
+
+;;; Generate a reference to a manifest constant, creating a new leaf
+;;; if necessary. If we are producing a fasl file, make sure that
+;;; MAKE-LOAD-FORM gets used on any parts of the constant that it
+;;; needs to be.
+(defun reference-constant (start next result value)
+  (declare (type ctran start next)
+           (type (or lvar null) result))
+  (ir1-error-bailout (start next result value)
+    (when (producing-fasl-file)
+      (maybe-emit-make-load-forms value))
+    (let* ((leaf (find-constant value))
+           (res (make-ref leaf)))
+      (push res (leaf-refs leaf))
+      (link-node-to-previous-ctran res start)
+      (use-continuation res next result)))
+  (values))
 
 ;;; Add FUNCTIONAL to the COMPONENT-REANALYZE-FUNCTIONALS, unless it's
 ;;; some trivial type for which reanalysis is a trivial no-op.
@@ -739,9 +766,14 @@
         ;; error (bug 412, lp#722734): checking for null RESULT is not enough,
         ;; since variables can become dead due to later optimizations.
         (ir1-convert start next result
-                     (if (eq (global-var-kind var) :global)
-                         `(sym-global-val ',name)
-                         `(symeval ',name)))
+                     (case (global-var-kind var)
+                       (:global `(sym-global-val ',name))
+                       (:unknown
+                        (when (not (deprecated-thing-p 'variable name))
+                          (note-undefined-reference name :variable))
+                        `(symeval ',name))
+                       (t
+                        `(symeval ',name))))
         (etypecase var
           (leaf
            (cond
@@ -951,9 +983,10 @@
 
 ;;; Convert a bunch of forms, discarding all the values except the
 ;;; last. If there aren't any forms, then translate a NIL.
-(declaim (ftype (sfunction (ctran ctran (or lvar null) list) (values))
-                ir1-convert-progn-body))
 (defun ir1-convert-progn-body (start next result body)
+  (declare (type ctran start next)
+           (type (or lvar null) result)
+           (type list body))
   (if (endp body)
       (reference-constant start next result nil)
       (let ((this-start start)
@@ -972,53 +1005,6 @@
   (values))
 
 
-;;;; code coverage
-
-;;; Check the policy for whether we should generate code coverage
-;;; instrumentation. If not, just return the original START
-;;; ctran. Otherwise insert code coverage instrumentation after
-;;; START, and return the new ctran.
-(defun instrument-coverage (start mode form
-                            &aux (metadata (coverage-metadata *compilation*)))
-  ;; We don't actually use FORM for anything, it's just convenient to
-  ;; have around when debugging the instrumentation.
-  (declare (ignore form))
-  (if (and metadata
-           (policy *lexenv* (> store-coverage-data 0))
-           *allow-instrumenting*)
-      (let ((path (source-path-original-source *current-path*)))
-        (when mode
-          (push mode path))
-        (if (member (ctran-block start)
-                    (gethash path (code-coverage-blocks metadata)))
-            ;; If this source path has already been instrumented in
-            ;; this block, don't instrument it again.
-            start
-            (let ((next (make-ctran))
-                  (*allow-instrumenting* nil))
-              (ensure-gethash path (code-coverage-records metadata)
-                              (cons path +code-coverage-unmarked+))
-              (push (ctran-block start)
-                    (gethash path (code-coverage-blocks metadata)))
-              (ir1-convert start next nil `(%primitive mark-covered ',path))
-              next)))
-      start))
-
-;;; In contexts where we don't have a source location for FORM
-;;; e.g. due to it not being a cons, but where we have a source
-;;; location for the enclosing cons, use the latter source location if
-;;; available. This works pretty well in practice, since many PROGNish
-;;; macroexpansions will just directly splice a block of forms into
-;;; some enclosing form with `(progn ,@body), thus retaining the
-;;; EQness of the conses.
-(defun maybe-instrument-progn-like (start forms form)
-  (or (when (and *allow-instrumenting*
-                 (not (get-source-path form)))
-        (let ((*current-path* (get-source-path forms)))
-          (when *current-path*
-            (instrument-coverage start nil form))))
-      start))
-
 ;;;; converting combinations
 
 ;;; Does this form look like something that we should add single-stepping
@@ -1047,9 +1033,12 @@
 ;;; Convert a function call where the function FUN is a LEAF. FORM is
 ;;; the source for the call. We return the COMBINATION node so that
 ;;; the caller can poke at it if it wants to.
-(declaim (ftype (sfunction (ctran ctran (or lvar null) list leaf) combination)
-                ir1-convert-combination))
 (defun ir1-convert-combination (start next result form fun)
+  (declare (type ctran start next)
+           (type (or lvar null) result)
+           (type list form)
+           (type leaf fun)
+           #-sb-xc-host (values combination))
   (let ((ctran (make-ctran))
         (fun-lvar (make-lvar)))
     (ir1-convert start ctran fun-lvar `(the (or function symbol) ,fun))
@@ -1076,7 +1065,7 @@
   (declare (type ctran start next)
            (type lvar fun-lvar)
            (type (or lvar null) result)
-           (list args))
+           (type list args))
   (let ((node (make-combination fun-lvar)))
     (setf (lvar-dest fun-lvar) node)
     (collect ((arg-lvars))
@@ -1101,15 +1090,6 @@
         (setf (combination-args node) (arg-lvars))))
     node))
 
-(defun show-transform-p (showp fun-name)
-  (or (and (listp showp) (member fun-name showp :test 'equal))
-      (eq showp t)))
-(defun show-transform (kind name new-form)
-  (let ((*print-length* 100)
-        (*print-level* 50)
-        (*print-right-margin* 128))
-    (format *trace-output* "~&xform (~a) ~S~% -> ~S~%"
-            kind name new-form)))
 ;;; Convert a call to a global function. If not NOTINLINE, then we do
 ;;; source transforms and try out any inline expansion. If there is no
 ;;; expansion, but is INLINE, then give an efficiency note (unless a
@@ -1139,7 +1119,9 @@
                          (record-call name (ctran-block start) *current-path*))
                        (when (show-transform-p *show-transforms-p* name)
                          (show-transform "src" name transformed))
-                       (let ((*transforming* t))
+                       (let ((*transforming* (1+ *transforming*)))
+                         (when (eq (car *current-path*) 'original-source-start)
+                           (setf (ctran-source-path start) *current-path*))
                          (ir1-convert start next result transformed)))))
               (ir1-convert-maybe-predicate start next result
                                            (proper-list form)
@@ -1209,7 +1191,6 @@
 
 ;;;; PROCESS-DECLS
 
-#-sb-devel
 (declaim (start-block process-decls make-new-inlinep
                       find-in-bindings
                       process-muffle-decls
@@ -1219,9 +1200,9 @@
 ;;; LAMBDA-VAR for that name, or NIL if it isn't found. We return the
 ;;; *last* variable with that name, since LET* bindings may be
 ;;; duplicated, and declarations always apply to the last.
-(declaim (ftype (sfunction (list symbol) (or lambda-var list))
-                find-in-bindings))
 (defun find-in-bindings (vars name)
+  (declare (list vars) (symbol name)
+           #-sb-xc-host (values (or lambda-var list)))
   (let ((found nil))
     (dolist (var vars)
       (cond ((leaf-p var)
@@ -1674,6 +1655,18 @@
         (make-lexenv
          :default res
          :flushable (cdr spec)))
+       (current-defmethod
+        (destructuring-bind (name qualifiers specializers lambda-list)
+            (cdr spec)
+          (let* ((gfs (or *methods-in-compilation-unit*
+                          (setf *methods-in-compilation-unit*
+                                (make-hash-table :test #'equal))))
+                 (methods (or (gethash name gfs)
+                              (setf (gethash name gfs)
+                                    (make-hash-table :test #'equal)))))
+            (setf (gethash (cons qualifiers specializers) methods)
+                  lambda-list)))
+        res)
        (no-constraints
         (process-no-constraints-decl spec vars)
         res)
@@ -1723,6 +1716,7 @@
         (allow-explicit-check allow-lambda-list)
         (lambda-list (if allow-lambda-list :unspecified nil))
         (optimize-qualities)
+        (local-optimize)
         source-form
         (post-binding-lexenv (if binding-form-p (list nil)))) ; dummy cell
     (flet ((process-it (spec decl)
@@ -1765,6 +1759,12 @@
                    ((equal spec '(top-level-form))) ; ignore
                    ((typep spec '(cons (eql source-form)))
                     (setf source-form (cadr spec)))
+                   ;; Used only for the current function.
+                   ;; E.g. suppressing argument checking without doing
+                   ;; so in all the subforms.
+                   ((typep spec '(cons (eql local-optimize)))
+                    (setf local-optimize spec)
+                    (setf source-form (cadr spec)))
                    (t
                     (multiple-value-bind (new-env new-qualities)
                         (process-1-decl spec lexenv vars fvars
@@ -1780,8 +1780,11 @@
               ;; Kludge: EVAL calls this function to deal with LOCALLY.
               (process-it spec decl)))))
     (warn-repeated-optimize-qualities (lexenv-policy lexenv) optimize-qualities)
+
     (values lexenv result-type (cdr post-binding-lexenv)
-            lambda-list explicit-check source-form)))
+            lambda-list explicit-check source-form
+            (when local-optimize
+              (process-optimize-decl local-optimize (lexenv-policy lexenv))))))
 
 (defun %processing-decls (decls vars fvars ctran lvar binding-form-p fun)
   (multiple-value-bind (*lexenv* result-type post-binding-lexenv)
