@@ -39,13 +39,13 @@
 ;;; them at a known location.
 (defun make-old-fp-save-location (env)
   (specify-save-tn
-   (physenv-debug-live-tn (make-normal-tn *fixnum-primitive-type*) env)
+   (environment-debug-live-tn (make-normal-tn *fixnum-primitive-type*) env)
    (make-wired-tn *fixnum-primitive-type*
                   control-stack-arg-scn
                   ocfp-save-offset)))
 (defun make-return-pc-save-location (env)
   (specify-save-tn
-   (physenv-debug-live-tn (make-normal-tn *backend-t-primitive-type*) env)
+   (environment-debug-live-tn (make-normal-tn *backend-t-primitive-type*) env)
    (make-wired-tn *backend-t-primitive-type*
                   control-stack-arg-scn
                   lra-save-offset)))
@@ -115,7 +115,7 @@
     (emit-label start-lab)
     ;; Allocate function header.
     (inst simple-fun-header-word)
-    (inst .skip (* (1- simple-fun-code-offset) n-word-bytes))
+    (inst .skip (* (1- simple-fun-insts-offset) n-word-bytes))
     (let ((entry-point (gen-label)))
       (emit-label entry-point)
       ;; FIXME alpha port has a ### note here saying we should "save it
@@ -142,7 +142,7 @@
     (move res csp-tn)
     (inst addi csp-tn csp-tn
           (* n-word-bytes (sb-allocated-size 'control-stack)))
-    (when (ir2-physenv-number-stack-p callee)
+    (when (ir2-environment-number-stack-p callee)
       (let* ((nbytes (bytes-needed-for-non-descriptor-stack-frame)))
         (when (> nbytes number-stack-displacement)
           (inst stdu nsp-tn nsp-tn (- (bytes-needed-for-non-descriptor-stack-frame)))
@@ -354,8 +354,7 @@ default-value-8
               values-start)
   (:temporary (:sc any-reg :offset nargs-offset
                :from :eval :to (:result 1))
-              nvals)
-  (:temporary (:scs (non-descriptor-reg)) temp))
+              nvals))
 
 
 ;;; This hook in the codegen pass lets us insert code before fall-thru entry
@@ -664,6 +663,12 @@ default-value-8
                      15
                      (if (eq return :unknown) 25 0))
 
+       ,@(when (eq named t)
+           '((aver (sc-is name constant))
+             ;; It has to be an fdefinition constant, not a normal IR1 constant.
+             ;; This is the best we can assert without peeking into IR2-COMPONENT-CONSTANTS.
+             (aver (not (sb-c::tn-leaf name)))))
+
        (let* ((cur-nfp (current-nfp-tn vop))
               ,@(unless (eq return :tail)
                   '((lra-label (gen-label))))
@@ -736,9 +741,6 @@ default-value-8
                   ;; Conditionally insert a conditional trap:
                   (when step-instrumenting
                     ;; Get the symbol-value of SB-IMPL::*STEPPING*
-                    #-sb-thread
-                    (load-symbol-value stepping sb-impl::*stepping*)
-                    #+sb-thread
                     (loadw stepping thread-base-tn thread-stepping-slot)
                     (inst cmpwi stepping 0)
                     ;; If it's not null, trap.
@@ -762,8 +764,7 @@ default-value-8
                      (loadw name-pass cfp-tn (tn-offset name))
                      (do-next-filler))
                     (constant
-                     (loadw name-pass code-tn (tn-offset name)
-                         other-pointer-lowtag)
+                     (loadw name-pass code-tn (tn-offset name) code-tn-lowtag)
                      (do-next-filler)))
                   ;; The step instrumenting must be done after
                   ;; FUNCTION is loaded, but before ENTRY-POINT is
@@ -776,10 +777,9 @@ default-value-8
                   ;; pointer) that the function itself be in a
                   ;; register before the raw-addr is loaded.
                   (sb-assem:without-scheduling ()
-                    (loadw function name-pass fdefn-fun-slot
-                        other-pointer-lowtag)
-                    (loadw entry-point name-pass fdefn-raw-addr-slot
-                        other-pointer-lowtag))
+                    (let ((tag (+ #-untagged-fdefns other-pointer-lowtag)))
+                      (loadw function name-pass fdefn-fun-slot tag)
+                      (loadw entry-point name-pass fdefn-raw-addr-slot tag)))
                   (do-next-filler)))
                ((nil)
                 `((sc-case arg-fun
@@ -788,8 +788,7 @@ default-value-8
                      (loadw lexenv cfp-tn (tn-offset arg-fun))
                      (do-next-filler))
                     (constant
-                     (loadw lexenv code-tn (tn-offset arg-fun)
-                         other-pointer-lowtag)
+                     (loadw lexenv code-tn (tn-offset arg-fun) code-tn-lowtag)
                      (do-next-filler)))
                   (loadw function lexenv closure-fun-slot
                       fun-pointer-lowtag)
@@ -799,7 +798,7 @@ default-value-8
                   ;; is calculated.
                   (insert-step-instrumenting function)
                   (inst addi entry-point function
-                        (- (ash simple-fun-code-offset word-shift)
+                        (- (ash simple-fun-insts-offset word-shift)
                            fun-pointer-lowtag))))
                (:direct
                 ;; Load fdefn-raw-address using NIL as the base pointer.
@@ -874,7 +873,7 @@ default-value-8
         (inst addi nsp-tn cur-nfp
               (- (bytes-needed-for-non-descriptor-stack-frame)
                  number-stack-displacement))))
-    (inst lr temp (make-fixup 'tail-call-variable :assembly-routine))
+    (inst addi temp null-tn (make-fixup 'tail-call-variable :asm-routine-nil-offset))
     (inst mtlr temp)
     (inst blr)))
 
@@ -1002,7 +1001,7 @@ default-value-8
       (move old-fp old-fp-arg)
       (move vals vals-arg)
       (move nvals nvals-arg)
-      (inst lr temp (make-fixup 'return-multiple :assembly-routine))
+      (inst addi temp null-tn (make-fixup 'return-multiple :asm-routine-nil-offset))
       (inst mtlr temp)
       (inst blr))))
 
@@ -1131,8 +1130,23 @@ default-value-8
   (:variant 0 0)
   (:translate %more-arg))
 
+(define-vop (more-arg-or-nil)
+  (:policy :fast-safe)
+  (:args (object :scs (descriptor-reg) :to (:result 1))
+         (count :scs (any-reg)))
+  (:info index)
+  (:results (value :scs (descriptor-reg any-reg)))
+  (:result-types *)
+  (:generator 3
+    (inst cmpwi count (fixnumize index))
+    (move value null-tn)
+    (inst ble done)
+    (loadw value object index)
+    done))
+
 ;;; Turn more arg (context, count) into a list.
-(define-vop (listify-rest-args)
+(define-vop ()
+  (:translate %listify-rest-args)
   (:args (context-arg :target context :scs (descriptor-reg))
          (count-arg :target count :scs (any-reg)))
   (:arg-types * tagged-num)
@@ -1142,7 +1156,6 @@ default-value-8
   (:temporary (:scs (non-descriptor-reg) :from :eval) dst)
   (:temporary (:sc non-descriptor-reg :offset nl3-offset) pa-flag)
   (:results (result :scs (descriptor-reg)))
-  (:translate %listify-rest-args)
   (:policy :safe)
   (:node-var node)
   (:generator 20
@@ -1158,7 +1171,7 @@ default-value-8
       (inst beq done)
 
     ;; We need to do this atomically.
-    (pseudo-atomic (pa-flag)
+    (pseudo-atomic (pa-flag :sync nil)
       ;; Allocate a cons (2 words) for each item.
       (if dx-p
           (progn
@@ -1170,7 +1183,7 @@ default-value-8
             (inst add csp-tn csp-tn temp))
           (progn
             (inst sldi temp count (1+ (- word-shift n-fixnum-tag-bits)))
-            (allocation result temp list-pointer-lowtag
+            (allocation 'list temp list-pointer-lowtag result
                         :temp-tn dst
                         :flag-tn pa-flag)
             (move dst result)))
@@ -1207,7 +1220,7 @@ default-value-8
 ;;; preventing this info from being returned as values.  What we do is
 ;;; compute (- SUPPLIED FIXED), and return a pointer that many words
 ;;; below the current stack top.
-(define-vop (more-arg-context)
+(define-vop ()
   (:policy :fast-safe)
   (:translate sb-c::%more-arg-context)
   (:args (supplied :scs (any-reg)))
@@ -1249,9 +1262,6 @@ default-value-8
   (:vop-var vop)
   (:generator 3
     ;; Get the symbol-value of SB-IMPL::*STEPPING*
-    #-sb-thread
-    (load-symbol-value stepping sb-impl::*stepping*)
-    #+sb-thread
     (loadw stepping thread-base-tn thread-stepping-slot)
     (inst cmpwi stepping 0)
     ;; If it's not zero, trap.
