@@ -12,33 +12,25 @@
 (in-package "SB-VM")
 
 
-(define-vop (list-or-list*)
-  (:args (things :more t))
+(define-vop (list)
+  (:args (things :more t :scs (any-reg descriptor-reg control-stack)))
   (:temporary (:scs (descriptor-reg)) ptr)
   (:temporary (:scs (descriptor-reg) :to (:result 0) :target result)
               res)
   (:temporary (:scs (any-reg)) temp)
   (:temporary (:sc non-descriptor-reg) pa-flag)
-  (:info num)
+  (:info star cons-cells)
   (:results (result :scs (descriptor-reg)))
-  (:policy :fast-safe)
-  (:variant-vars star)
   (:node-var node)
   (:generator 0
-    (cond ((zerop num)
-           (move result null-tn))
-          ((and star (= num 1))
-           (move result (tn-ref-tn things)))
-          (t (flet ((maybe-load (tn)
-                      (sc-case tn
-                        ((any-reg descriptor-reg)
-                         tn)
-                        (control-stack
-                         (load-stack-tn temp tn)
-                         temp))))
-               (let* ((cons-cells (if star (1- num) num))
-                      (alloc (* (pad-data-block cons-size) cons-cells)))
-                 (pseudo-atomic (pa-flag)
+    (flet ((maybe-load (tn)
+             (sc-case tn
+              ((any-reg descriptor-reg) tn)
+              (control-stack
+               (load-stack-tn temp tn)
+               temp))))
+      (let ((alloc (* (pad-data-block cons-size) cons-cells)))
+        (pseudo-atomic (pa-flag)
                    (allocation 'list alloc list-pointer-lowtag res
                                :flag-tn pa-flag
                                :stack-allocate-p (node-stack-allocate-p node)
@@ -57,13 +49,7 @@
                                (maybe-load (tn-ref-tn (tn-ref-across things)))
                                null-tn)
                        ptr cons-cdr-slot list-pointer-lowtag))
-                 (move result res)))))))
-
-(define-vop (list list-or-list*)
-  (:variant nil))
-(define-vop (list* list-or-list*)
-  (:variant t))
-
+        (move result res)))))
 
 ;;;; Special purpose inline allocators.
 
@@ -93,11 +79,8 @@
   (:policy :fast-safe)
   (:generator 100
     (pseudo-atomic (pa-flag)
-      (cond ((zerop (- word-shift n-fixnum-tag-bits))
-             (inst addi bytes words (* (1+ vector-data-offset) n-word-bytes)))
-            (t
-             (inst slli bytes words (- word-shift n-fixnum-tag-bits))
-             (inst addi bytes bytes (* (1+ vector-data-offset) n-word-bytes))))
+      (with-fixnum-as-word-index (words bytes)
+        (inst addi bytes words (* (1+ vector-data-offset) n-word-bytes)))
       (inst andi bytes bytes (lognot lowtag-mask))
       (allocation nil bytes other-pointer-lowtag result :flag-tn pa-flag)
       (storew type result 0 other-pointer-lowtag)
@@ -116,11 +99,8 @@
   (:policy :fast-safe)
   (:generator 100
     (pseudo-atomic (pa-flag)
-      (cond ((zerop (- word-shift n-fixnum-tag-bits))
-             (inst addi bytes words (* (1+ vector-data-offset) n-word-bytes)))
-            (t
-             (inst slli bytes words (- word-shift n-fixnum-tag-bits))
-             (inst addi bytes bytes (* (1+ vector-data-offset) n-word-bytes))))
+      (with-fixnum-as-word-index (words bytes)
+        (inst addi bytes words (* (1+ vector-data-offset) n-word-bytes)))
       (inst andi bytes bytes (lognot lowtag-mask))
 
       ;; FIXME: It would be good to check for stack overflow here.
@@ -207,12 +187,9 @@
   (:temporary (:sc non-descriptor-reg) header)
   (:results (result :scs (descriptor-reg)))
   (:generator 6
-    (cond ((zerop (- word-shift n-fixnum-tag-bits))
-           (inst addi bytes extra (* (1- words) n-word-bytes)))
-          (t
-           (inst slli bytes extra (- word-shift n-fixnum-tag-bits))
-           (inst addi bytes bytes (* (1- words) n-word-bytes))))
-    (inst slli header bytes (- n-widetag-bits word-shift))
+    (with-fixnum-as-word-index (extra bytes)
+      (inst addi bytes extra (* (1- words) n-word-bytes)))
+    (inst slli header bytes (- (length-field-shift type) word-shift))
     (inst addi header header type)
     ;; Add the object header to the allocation size and round up to
     ;; the allocation granularity.
