@@ -945,3 +945,14 @@
          (setf (ldb (byte 7 25) (sap-ref-32 sap offset))
                (ldb (byte 7 5) i))))))
    nil)
+
+(define-instruction store-coverage-mark (segment mark-index)
+  (:emitter
+   ;; No backpatch is needed to compute the offset into the code header
+   ;; because COMPONENT-HEADER-LENGTH is known at this point.
+   (let ((offset (+ (component-header-length)
+                    n-word-bytes ; skip over jump table word
+                    mark-index
+                    (- other-pointer-lowtag))))
+     (inst* segment 'sb sb-vm::null-tn sb-vm::code-tn
+            (the (unsigned-byte 11) offset)))))
