@@ -32,11 +32,11 @@
        ,inner (progn ,@body)
               (if (setq ,tn (tn-next ,tn)) (go ,inner) (go ,outer)))))
 
-(defun set-ir2-physenv-live-tns (value instance)
-  (setf (ir2-physenv-live-tns instance) value))
+(defun set-ir2-environment-live-tns (value instance)
+  (setf (ir2-environment-live-tns instance) value))
 
-(defun set-ir2-physenv-debug-live-tns (value instance)
-  (setf (ir2-physenv-debug-live-tns instance) value))
+(defun set-ir2-environment-debug-live-tns (value instance)
+  (setf (ir2-environment-debug-live-tns instance) value))
 
 (defun set-ir2-component-alias-tns (value instance)
   (setf (ir2-component-alias-tns instance) value))
@@ -81,14 +81,14 @@
                (case (tn-kind tn)
                  (:environment
                   (clear-live tn
-                              #'ir2-physenv-live-tns
-                              #'set-ir2-physenv-live-tns))
+                              #'ir2-environment-live-tns
+                              #'set-ir2-environment-live-tns))
                  (:debug-environment
                   (clear-live tn
-                              #'ir2-physenv-debug-live-tns
-                              #'set-ir2-physenv-debug-live-tns))))
+                              #'ir2-environment-debug-live-tns
+                              #'set-ir2-environment-debug-live-tns))))
              (clear-live (tn getter setter)
-               (let ((env (physenv-info (tn-physenv tn))))
+               (let ((env (environment-info (tn-environment tn))))
                  (funcall setter (delete tn (funcall getter env)) env))))
       (declare (inline used-p delete-some delete-1 clear-live))
       (delete-some #'ir2-component-alias-tns
@@ -170,24 +170,24 @@
   (make-tn (incf (ir2-component-global-tn-counter (component-info *component-being-compiled*)))
            :unused nil nil))
 
-;;; Make TN be live throughout PHYSENV. Return TN. In the DEBUG case,
-;;; the TN is treated normally in blocks in the environment which
+;;; Make TN be live throughout ENV. Return TN. In the DEBUG case, the
+;;; TN is treated normally in blocks in the environment which
 ;;; reference the TN, allowing targeting to/from the TN. This results
 ;;; in move efficient code, but may result in the TN sometimes not
 ;;; being live when you want it.
-(defun physenv-live-tn (tn physenv)
-  (declare (type tn tn) (type physenv physenv))
+(defun environment-live-tn (tn env)
+  (declare (type tn tn) (type environment env))
   (aver (eq (tn-kind tn) :normal))
   (setf (tn-kind tn) :environment)
-  (setf (tn-physenv tn) physenv)
-  (push tn (ir2-physenv-live-tns (physenv-info physenv)))
+  (setf (tn-environment tn) env)
+  (push tn (ir2-environment-live-tns (environment-info env)))
   tn)
-(defun physenv-debug-live-tn (tn physenv)
-  (declare (type tn tn) (type physenv physenv))
+(defun environment-debug-live-tn (tn env)
+  (declare (type tn tn) (type environment env))
   (aver (eq (tn-kind tn) :normal))
   (setf (tn-kind tn) :debug-environment)
-  (setf (tn-physenv tn) physenv)
-  (push tn (ir2-physenv-debug-live-tns (physenv-info physenv)))
+  (setf (tn-environment tn) env)
+  (push tn (ir2-environment-debug-live-tns (environment-info env)))
   tn)
 
 ;;; Make TN be live throughout the current component. Return TN.
@@ -216,11 +216,15 @@
 ;;; Create a constant TN. The backend dependent
 ;;; IMMEDIATE-CONSTANT-SC function is used to determine whether the
 ;;; constant has an immediate representation.
+;;; FIXME: this can create multiple boxed TNs (consuming more than one
+;;; slot in the boxed constant vector) for a given leaf when called by
+;;; EMIT-MOVES-AND-COERCIONS. That's wasteful.
 (defun make-constant-tn (constant &optional force-boxed)
   (declare (type constant constant))
   (or (leaf-info constant)
       (multiple-value-bind (immed null-offset)
           (immediate-constant-sc (constant-value constant))
+        ;; currently NULL-OFFSET is used only on ARM64
         (if null-offset
             (setf (leaf-info constant)
                   (component-live-tn
@@ -248,7 +252,7 @@
                              #+immobile-space
                              (let ((val (constant-value constant)))
                                (or (and (symbolp val) (not (sb-vm:static-symbol-p val)))
-                                   (typep val 'layout)))))
+                                   (typep val 'wrapper)))))
                 (let ((constants (ir2-component-constants component)))
                   (setf (tn-offset res)
                         (vector-push-extend constant constants))))
@@ -259,6 +263,23 @@
               (setf (tn-leaf res) constant)
               res)))))
 
+;;; Extracted from above
+(defun constant-sc (constant)
+  (if (leaf-info constant)
+      (tn-sc (leaf-info constant))
+      (multiple-value-bind (immed null-offset)
+          (immediate-constant-sc (constant-value constant))
+        (if null-offset
+            immed
+            (let ((boxed (or (not immed)
+                             (boxed-immediate-sc-p immed))))
+              (cond (boxed
+                     (if immed
+                         (svref *backend-sc-numbers* immed)
+                         (sc-or-lose 'constant)))
+                    (t
+                     (sc-or-lose 'constant))))))))
+
 (defun make-load-time-value-tn (handle type)
   (let* ((component (component-info *component-being-compiled*))
          (sc (svref *backend-sc-numbers*
@@ -286,6 +307,11 @@
 ;;; Return a load-time constant TN with the specified KIND and INFO.
 ;;; If the desired CONSTANTS entry already exists, then reuse it,
 ;;; otherwise allocate a new load-time constant slot.
+;;; FIXME: this function deserves a comment about why it creates a new TN even
+;;; when KIND + INFO are already found. It's as if we need to maintain a 1:1
+;;; relation between TN and TN-REF for a subset of constants, but it's unclear
+;;; why that should be necessary. The shape of this algorithm suggests more than
+;;; mere oversight, and that it was a deliberate choice.
 (defun make-load-time-constant-tn (kind info)
   (declare (type keyword kind))
   (let* ((component (component-info *component-being-compiled*))
@@ -295,7 +321,7 @@
                        (svref *backend-sc-numbers* sb-vm:constant-sc-number)))
          (constants (ir2-component-constants component)))
     (setf (tn-type res) *universal-type*)
-    (do ((i 0 (1+ i)))
+    (do ((i 1 (1+ i)))
         ((= i (length constants))
          (setf (tn-offset res) i)
          (vector-push-extend (list kind info res) constants))
@@ -328,6 +354,18 @@
           (push-in tn-ref-next res (tn-reads tn))))
     res))
 
+(defun reference-tn-refs (refs write-p)
+  (when refs
+    (let* ((first (reference-tn (tn-ref-tn refs) write-p) )
+           (prev first))
+      (loop for tn-ref = (tn-ref-across refs) then (tn-ref-across tn-ref)
+            while tn-ref
+            do
+            (let ((ref (reference-tn  (tn-ref-tn tn-ref) write-p)))
+              (setf (tn-ref-across prev) ref)
+              (setq prev ref)))
+      first)))
+
 ;;; Make TN-REFS to reference each TN in TNs, linked together by
 ;;; TN-REF-ACROSS. WRITE-P is the WRITE-P value for the refs. MORE is
 ;;; stuck in the TN-REF-ACROSS of the ref for the last TN, or returned
