@@ -316,7 +316,9 @@
        (cond ((and (eql (numeric-type-class type) 'integer)
                    (and (fixnump low)
                         (fixnump high)
-                        (<= (1+ (- high low)) 2)))
+                        #+(or x86 x86-64 arm arm64)
+                        (/= low 0)
+                        (< (- high low) 2)))
               ;; The fixnum-mod-p case is worse than just EQ testing with
               ;; only 2 values in the range. (INTEGER 1 2) would have become
               ;;   (and (not (eq x 0)) (fixnump x) (not (> x 2))).
@@ -365,7 +367,7 @@
            ;; as soon as any unknown is present.
            `(classoid-cell-typep ,(find-classoid-cell spec :create t) ,object))
           ((unknown-type-p type)
-           #+sb-xc-host
+           #+(and sb-xc-host (not sb-devel))
            (warn "can't open-code test of unknown type ~S"
                  (type-specifier type))
            ;; This is not a policy-based decision to notify here,
@@ -538,13 +540,21 @@
           (t
            (multiple-value-bind (widetags more-types)
                (sb-kernel::widetags-from-union-type types)
-             (if widetags
-                 `(or (%other-pointer-subtype-p ,object ',widetags)
-                      (typep ,object '(or ,@(mapcar #'type-specifier more-types))))
-                 `(or
-                   ,@(mapcar (lambda (x)
-                               `(typep ,object ',(type-specifier x)))
-                             more-types))))))))
+             (multiple-value-bind (predicate more-union-types)
+                 (split-union-type-tests type)
+               (cond ((and predicate
+                           (< (length more-union-types)
+                              (length more-types)))
+                      `(or (,predicate ,object)
+                           (typep ,object '(or ,@(mapcar #'type-specifier more-union-types)))))
+                     (widetags
+                      `(or (%other-pointer-subtype-p ,object ',widetags)
+                           (typep ,object '(or ,@(mapcar #'type-specifier more-types)))))
+                     (t
+                      `(or
+                        ,@(mapcar (lambda (x)
+                                    `(typep ,object ',(type-specifier x)))
+                                  more-types))))))))))
 
 ;;; Do source transformation for TYPEP of a known intersection type.
 (defun source-transform-intersection-typep (object type)
@@ -665,22 +675,26 @@
 ;;;
 ;;; Secondary return value is true if passing the generated tests implies that
 ;;; the array has a header.
-(defun test-array-dimensions (obj type stype
+(defun test-array-dimensions (original-obj type stype
                               simple-array-header-p)
   (declare (type array-type type stype))
-  (let ((obj `(truly-the ,(type-specifier stype) ,obj))
+  (let ((obj `(truly-the ,(type-specifier stype) ,original-obj))
         (dims (array-type-dimensions type))
         (header-test (if simple-array-header-p
-                         `(simple-array-header-p ,obj)
-                         `(array-header-p ,obj))))
+                         `(simple-array-header-p ,original-obj)
+                         `(array-header-p ,original-obj))))
     (unless (or (eq dims '*)
                 (equal dims (array-type-dimensions stype)))
       (cond ((cdr dims)
-             (values `(,header-test
-                       ,@(when (eq (array-type-dimensions stype) '*)
-                           (if (vop-existsp :translate %array-rank=)
-                               `((%array-rank= ,obj ,(length dims)))
-                               `((= (%array-rank ,obj) ,(length dims)))))
+             (values `(,@(if (and simple-array-header-p
+                                  (vop-existsp :translate simple-array-header-of-rank-p)
+                                  (eq (array-type-dimensions stype) '*))
+                             `((simple-array-header-of-rank-p ,original-obj ,(length dims)))
+                             `(,header-test
+                               ,@(when (eq (array-type-dimensions stype) '*)
+                                   (if (vop-existsp :translate %array-rank=)
+                                       `((%array-rank= ,obj ,(length dims)))
+                                       `((= (%array-rank ,obj) ,(length dims)))))))
                        ,@(loop for d in dims
                                for i from 0
                                unless (eq '* d)
@@ -796,7 +810,7 @@
 ;;; layout-EQ. If a structure then test for layout-EQ and then a
 ;;; general test based on layout-inherits. Otherwise, look up the indirect
 ;;; class-cell and call CLASS-CELL-TYPEP at runtime.
-(deftransform %instance-typep ((object spec) (* *) * :node node)
+(deftransform %instance-typep ((object spec) * * :node node)
   (aver (constant-lvar-p spec))
   (let* ((spec (lvar-value spec))
          (class (specifier-type spec))
@@ -1180,6 +1194,35 @@
                     (error "~a is not a subtype of VECTOR." type)))))
     (simplify type)))
 
+(defun strip-array-dimensions-and-complexity (type)
+  (labels ((process-compound-type (types)
+             (let (array-types)
+               (dolist (type types)
+                 (unless (or (hairy-type-p type)
+                             (sb-kernel::negation-type-p type))
+                   (push (strip type) array-types)))
+               (apply #'type-union array-types)))
+           (strip (type)
+             (cond ((array-type-p type)
+                    (let ((dim (array-type-dimensions type)))
+                      (make-array-type
+                       (if (eq dim '*)
+                           dim
+                           (make-list (length dim)
+                                      :initial-element '*))
+                       :complexp :maybe
+                       :element-type (array-type-element-type type)
+                       :specialized-element-type (array-type-specialized-element-type type))))
+                   ((union-type-p type)
+                    (process-compound-type (union-type-types type)))
+                   ((intersection-type-p type)
+                    (process-compound-type (intersection-type-types type)))
+                   ((member-type-p type)
+                    (process-compound-type
+                     (mapcar #'ctype-of (member-type-members type))))
+                   (t
+                    (error "~a is not a subtype of ARRAY." type)))))
+    (strip type)))
 
 (defun check-coerce (value-type to-type type-specifier node)
   (flet ((fail ()
@@ -1214,7 +1257,7 @@
           (t
            (fail)))))
 
-(deftransform coerce ((x type) (* *) * :node node)
+(deftransform coerce ((x type) * * :node node)
   (unless (constant-lvar-p type)
     (give-up-ir1-transform))
   (let* ((tval (lvar-value type))
