@@ -33,7 +33,7 @@ os_vm_address_t arch_get_bad_addr(int sig, siginfo_t *code, os_context_t *contex
     unsigned int *pc;
     int rs1;
 
-    pc = (unsigned int *)(*os_context_pc_addr(context));
+    pc = (unsigned int *)OS_CONTEXT_PC(context);
 
     /* On the sparc, we have to decode the instruction. */
 
@@ -81,7 +81,7 @@ os_vm_address_t arch_get_bad_addr(int sig, siginfo_t *code, os_context_t *contex
 
 void arch_skip_instruction(os_context_t *context)
 {
-    *os_context_pc_addr(context) = *os_context_npc_addr(context);
+    OS_CONTEXT_PC(context) = *os_context_npc_addr(context);
     /* Note that we're doing integer arithmetic here, not pointer. So
      * the value that the return value of os_context_npc_addr() points
      * to will be incremented by 4, not 16.
@@ -91,7 +91,7 @@ void arch_skip_instruction(os_context_t *context)
 
 unsigned char *arch_internal_error_arguments(os_context_t *context)
 {
-    return (unsigned char *)(*os_context_pc_addr(context) + 4);
+    return (unsigned char *)(OS_CONTEXT_PC(context) + 4);
 }
 
 boolean arch_pseudo_atomic_atomic(os_context_t *context)
@@ -158,7 +158,7 @@ static unsigned int *skipped_break_addr, displaced_after_inst;
 
 void arch_do_displaced_inst(os_context_t *context, unsigned int orig_inst)
 {
-    unsigned int *pc = (unsigned int *)(*os_context_pc_addr(context));
+    unsigned int *pc = (unsigned int *)OS_CONTEXT_PC(context);
     unsigned int *npc = (unsigned int *)(*os_context_npc_addr(context));
 
   /*  orig_sigmask = context->sigmask;
@@ -182,7 +182,7 @@ static int pseudo_atomic_trap_p(os_context_t *context)
     int result;
 
 
-    pc = (unsigned int*) *os_context_pc_addr(context);
+    pc = (unsigned int*) OS_CONTEXT_PC(context);
     badinst = *pc;
     result = 0;
 
@@ -221,8 +221,8 @@ arch_handle_breakpoint(os_context_t *context)
 void
 arch_handle_fun_end_breakpoint(os_context_t *context)
 {
-    *os_context_pc_addr(context) = (int) handle_fun_end_breakpoint(context);
-    *os_context_npc_addr(context) = *os_context_pc_addr(context) + 4;
+    OS_CONTEXT_PC(context) = (int) handle_fun_end_breakpoint(context);
+    *os_context_npc_addr(context) = OS_CONTEXT_PC(context) + 4;
 }
 
 void
@@ -231,35 +231,36 @@ arch_handle_after_breakpoint(os_context_t *context)
     *skipped_break_addr = trap_Breakpoint;
     os_flush_icache(skipped_break_addr, sizeof(unsigned int));
     skipped_break_addr = NULL;
-    *(unsigned long *) os_context_pc_addr(context) = displaced_after_inst;
+    *(unsigned long *)OS_CONTEXT_PC(context) = displaced_after_inst;
     /* context->sigmask = orig_sigmask; */
-    os_flush_icache((os_vm_address_t) os_context_pc_addr(context), sizeof(unsigned int));
+    os_flush_icache((os_vm_address_t)OS_CONTEXT_PC(context), sizeof(unsigned int));
 }
 
 void
 arch_handle_single_step_trap(os_context_t *context, int trap)
 {
-    unsigned int code = *((u32 *)(*os_context_pc_addr(context)));
+    unsigned int code = *(uint32_t *)OS_CONTEXT_PC(context);
     int register_offset = code >> 8 & 0x1f;
     handle_single_step_trap(context, trap, register_offset);
     arch_skip_instruction(context);
 }
 
 #ifdef LISP_FEATURE_GENCGC
-void
-arch_handle_allocation_trap(os_context_t *context)
+static void handle_allocation_trap(os_context_t *context, unsigned int *pc)
 {
-    unsigned int* pc;
     unsigned int or_inst;
     int rs1;
     int size;
     int immed;
-    extern void boxed_region_rollback(sword_t);
+    extern void mixed_region_rollback(sword_t);
 
     if (foreign_function_call_active)
       lose("Allocation trap inside foreign code.");
 
-    pc = (unsigned int*) *os_context_pc_addr(context);
+    struct thread* thread = get_sb_vm_thread();
+    if (gencgc_alloc_profiler && thread->state_word.sprof_enable)
+        record_backtrace_from_context(context, thread);
+
     or_inst = pc[-1];
 
     /*
@@ -267,8 +268,8 @@ arch_handle_allocation_trap(os_context_t *context)
      * instruction!
      */
     if (!(((or_inst >> 30) == 2) && (((or_inst >> 19) & 0x1f) == 2)))
-        lose("Allocation trap not preceded by an OR instruction: 0x%08x",
-             or_inst);
+        lose("Allocation trap @ %p not preceded by an OR instruction: 0x%08x",
+             pc, or_inst);
 
     /*
      * An OR instruction.  RS1 is the register we want to allocate to.
@@ -282,7 +283,7 @@ arch_handle_allocation_trap(os_context_t *context)
         size = or_inst & 0x1fff;
     else
         size = *os_context_register_addr(context, or_inst & 0x1f);
-    boxed_region_rollback(size);
+    mixed_region_rollback(size);
 
     fake_foreign_function_call(context);
 
@@ -291,8 +292,7 @@ arch_handle_allocation_trap(os_context_t *context)
      */
     lispobj* memory;
     {
-        struct interrupt_data *data =
-            arch_os_get_current_thread()->interrupt_data;
+        struct interrupt_data *data = &thread_interrupt_data(thread);
         data->allocation_trap_context = context;
         extern lispobj *alloc(sword_t), *alloc_list(sword_t);
         memory = (rd & 1) ? alloc_list(size) : alloc(size);
@@ -314,9 +314,16 @@ static void sigill_handler(int signal, siginfo_t *siginfo,
         unsigned int inst;
         unsigned int* pc = (unsigned int*) siginfo->si_addr;
 
+        if (!gc_managed_heap_space_p((lispobj)pc))
+          lose("Illegal instruction not in lisp: %p [%x]\n", pc, *pc);
+
         inst = *pc;
         trap = inst & 0xff;
-        handle_trap(context,trap);
+#ifdef LISP_FEATURE_GENCGC
+        if (trap == trap_Allocation) handle_allocation_trap(context, pc);
+        else
+#endif
+            handle_trap(context,trap);
     }
     else if ((siginfo->si_code) == ILL_ILLTRP) {
         if (pseudo_atomic_trap_p(context)) {
